! ===========================================================================================================
! integer function read_meta.f90
! 
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20191126 VH     Save the launch date minus one year to test for bad gps fix
! 20190521 VH     Set number of greylist entries to 1 if an 'add to grey list' line has only 'n' not 'n 1' or 'n 2',
! 20190416 VH     Check for STS conductivity sensor (controls qc22, near-surface)
! 20180619 VH     New variable, float_type (first part of inst_type) to be used later
! 20171020 VH     Add ability to recognize new inst_type: ALAMO<>
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
! ===========================================================================================================

integer function read_meta(pathname)

use communication                       ! module
use data_arrays                         ! module

! Fri nit:  psal_adj - max recs for a float is SIX (3 floats)

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=*), intent(in)  :: pathname

character(len=40) :: testst		! test string to read launch
character(len=2)  :: testch		! 2ch
character(len=4)  :: test_params(4)	! parameters put on the grey list
character(len=16) :: float_mfrer        ! Float Manufacturer 
integer		  :: test_flags(4)	! what are the flags to use
logical :: wmoid_found,inst_type_found,trans_type_found,launch_time_found,launch_posn_found	! These are MANDATORY
logical :: send_GTS_found,grey_list_found,psal_adj_found			! These are OPTIONAL
real :: extravals(5)			! test nominal_profile_pres
real :: lat_deg, lat_min, lon_deg, lon_min
integer :: ib; ! String locator for inst_type_short
integer :: status,ios
logical :: float_mfrer_found

character(len=9), parameter :: sub = 'read_meta'
integer :: ldate(6),lyr,lmon,lday,lhh,lmm,lsc;
equivalence (ldate(1),lyr), (ldate(2),lmon),(ldate(3),lday),(ldate(4),lhh),(ldate(5),lmm),(ldate(6),lsc);

! ===========================================================================================================
! executable statements

read_meta = 1;				! Assume failure
!write(msg,*) "DBG:>>> Opening old file " ,pathname ; call wrtlog(sub,msg)

! Initialize grey list arrays (defined in data_arrays.f90
add_to_grey_list = .false.
do igp=1,MAX_GL_PARAM
    greyflag(igp) = 1;
    test_params(igp) = 'XXXX';
    test_flags(igp) = 1;
    do igl=1,MAX_GL
	param_on_grey_list(igl,igp) = .false.
	grey_flag(igl,igp) = 1 ! Use zero as place holder
    enddo
enddo

if(DBGIO) call dbg_enter(sub);

inp_open=open_old_file(INP,pathname,"INP","meta file");
write(msg,'("Read META: ",a)')trim(pathname); call wrtstd(sub,msg);

wmoid_found       = .false.
inst_type_found   = .false.
trans_type_found  = .false.
launch_time_found = .false.
launch_posn_found = .false.
sampling_found    = .false.
VERT_SAMPLING_SCHEME_FROM = "UNKNOWN"
do iprf=1,MAXPRF
    do_psal_prof(iprf) = .false.;		! initialize at beginning of read_meta
    chr_VERT_SAMPLING_SCHEME(iprf) = "n/a";	! initialize at beginning of read_meta
enddo
send_GTS_found    = .false.  ! This is optional - the line is not required in the meta file
send_to_GTS       = .true.   ! default is to send the profile
grey_list_found   = .false.  ! initialize at beginning of read_meta
psal_adj_found    = .false.  ! initialize at beginning of read_meta	
do_psal_adj       = .false.  ! initialize at beginning of read_meta
float_mfrer_found = .false.  ! initialize at beginning of read_meta

frozen_profile_rule = 0      ! How do we treat frozen profiles? (0=regular, 1=Gilson special)

maximum_prof_pres  = FILL;		! maximum expected profile pres (used in qc19)
maximum_prof_pres_from = "META_FILE";	!where did the max pres come from?

npa=0; ! Counts the PSAL ADJUST lines

nmet = 0;
read(INP,'(a)',iostat=ios) cline;
nmet=nmet+1; metline(nmet) = trim(cline);
do while (ios==IO_OK)
    if(len(trim(cline)) == 0) go to 10; ! SKIP PARSING, line is empty
    if(cline(1:30) == '==============================') go to 10; ! SKIP PARSING, line is =====s
    call check_hed(sub,pathname,cline); ! if check hed failf in read_meta, it is a FATAL ERROR
    read(cline,'(a40,a)')variable,value
    if(len_trim(variable)>0 .and. len_trim(value) == 0 .and. index(trim(variable), '================') == 0)then
	write(errmsg(1),'("NON-FATAL ERROR 232 in read_meta")')
	write(errmsg(2),'("NON-FATAL ERROR 232 No value found in meta file for line: ",a)')trim(cline);
	call error_report(sub,232,2,errmsg); ! read_meta No value found in meta file line
    endif
    if(trim(variable) == 'WMO ID number')then
	wmoid_found = .true.
	wmoid = trim(value);
    elseif(trim(variable) == 'instrument type     ')then
	inst_type_found = .true.
	inst_type = trim (value);
	float_type = inst_type(1:(index(inst_type,'_')-1));
    elseif(trim(variable) == 'float manufacturer    ')then
        float_mfrer_found = .true.
        float_mfrer = trim(value)
        if(inst_type_found)then
         call to_upper(float_mfrer)
         call get_platform_type(float_mfrer,chr_PLATFORM_TYPE) 
	endif
    elseif(trim(variable) == 'PI')then
	var_PI_NAME = trim(value)
	call to_upper(var_PI_NAME);
    elseif(trim(variable) == 'internal ID number')then
	var_INTERNAL_ID = trim(value)
    elseif(trim(variable) == 'transmission ID number')then
	var_TRANSMISSION_NO = trim(value)
    elseif(trim(variable) == 'ROM version')then
	var_FIRMWARE_VERSION = trim(value)
    elseif(trim(variable) == 'float serial number')then
	var_FLOAT_SERIAL_NO = trim(value);
    elseif(trim(variable) == 'WMO instrument type (table 1770)')then
	var_WMO_INST_TYPE = trim(value);
    elseif(trim(variable) == 'launch time [dd mm yyyy hh mm (Z)]')then
	launch_time_found = .true.
	testst = trim (value);
	! oops read(testst,'(i2,x,i2,x,i4,x,i2,x,i2)')launch_mon,launch_day,launch_year,launch_hh,launch_mm
	read(testst,'(i2,x,i2,x,i4,x,i2,x,i2)')launch_day,launch_mon,launch_year,launch_hh,launch_mm
	lyr=launch_year;lmon=launch_mon;lday=launch_day;lhh=launch_hh;lmm=launch_mm;lsc=0;
	badgps_test_lo = launch_year - 1;; ! Used to test for BADGPS
	launch_time_juld=juldays1950(ldate);
    else if(trim(variable) == 'launch position [lat latm lon lonm]')then
	launch_posn_found = .true.
	testst = value(1:40)
	read(testst,*) lat_deg,lat_min,lon_deg,lon_min
	launch_lat = lat_deg + (lat_min/60.0)
	launch_lon = lon_deg + (lon_min/60.0)
    else if((trim(variable)=='Profile Sampling Method') .or. (trim(variable)=='Profile Sampling Method(1)'))then
	sampling_found = .true.; sampling_entries = sampling_entries + 1;
	VERT_SAMPLING_SCHEME_FROM = "META_FILE";
	if(index(value,'mixed [') > 0)then
	    chr_VERT_SAMPLING_SCHEME(1) = 'Primary sampling: '//trim(value);
	elseif(index(value,'discrete [') > 0)then
	    chr_VERT_SAMPLING_SCHEME(1) = 'Primary sampling: '//trim(value);
	elseif(index(value,'averaged [') > 0)then
	    chr_VERT_SAMPLING_SCHEME(1) = 'Primary sampling: '//trim(value);
	elseif(trim(value)=='continuous'.or.trim(value)=='Continuous')then
	    chr_VERT_SAMPLING_SCHEME(1) = 'Primary sampling: averaged []';
	elseif(trim(value)=='discrete'.or.trim(value)=='Discrete')then
	    chr_VERT_SAMPLING_SCHEME(1) = 'Primary sampling: discrete []';
	elseif(index(value,'n/a') > 0)then
	    chr_VERT_SAMPLING_SCHEME(1) = 'Primary sampling: N/A';
	else
	    chr_VERT_SAMPLING_SCHEME(1) = 'Primary sampling: mixed []';
	endif
    else if(trim(variable) == 'Profile Sampling Method(2)')then
	sampling_found = .true.;  sampling_entries = sampling_entries + 1;
	VERT_SAMPLING_SCHEME_FROM = "META_FILE";
	if(index(value,'mixed [') > 0)then
	    chr_VERT_SAMPLING_SCHEME(2) = 'Secondary sampling: '//trim(value);
	elseif(index(value,'discrete [') > 0)then
	    chr_VERT_SAMPLING_SCHEME(2) = 'Secondary sampling: '//trim(value);
	elseif(index(value,'averaged [') > 0)then
	    chr_VERT_SAMPLING_SCHEME(2) = 'Secondary sampling: '//trim(value);
	elseif(trim(value)=='continuous'.or.trim(value)=='Continuous')then
	    chr_VERT_SAMPLING_SCHEME(2) = 'Secondary sampling: averaged []';
	elseif(trim(value)=='discrete'.or.trim(value)=='Discrete')then
	    chr_VERT_SAMPLING_SCHEME(2) = 'Secondary sampling: discrete []';
        elseif(index(value,"Near-surface sampling:") > 0)then
            chr_VERT_SAMPLING_SCHEME(2) = trim(value);  ! these are complete lines, including the "near-surface" part
	elseif(index(value,'n/a') > 0)then
	    chr_VERT_SAMPLING_SCHEME(2) = 'Secondary sampling: N/A';
	else
	    chr_VERT_SAMPLING_SCHEME(2) = 'Secondary sampling: mixed []';
	endif
    else if(trim(variable) == 'Profile Sampling Method(3)')then
	sampling_found = .true.;  sampling_entries = sampling_entries + 1;
	VERT_SAMPLING_SCHEME_FROM = "META_FILE";
	if(index(value,'mixed [') > 0)then
	    chr_VERT_SAMPLING_SCHEME(3) = 'Secondary sampling: '//trim(value);
	elseif(index(value,'discrete [') > 0)then
	    chr_VERT_SAMPLING_SCHEME(3) = 'Secondary sampling: '//trim(value);
	elseif(index(value,'averaged [') > 0)then
	    chr_VERT_SAMPLING_SCHEME(3) = 'Secondary sampling: '//trim(value);
	elseif(trim(value)=='continuous'.or.trim(value)=='Continuous')then
	    chr_VERT_SAMPLING_SCHEME(3) = 'Secondary sampling: averaged []';
	elseif(trim(value)=='discrete'.or.trim(value)=='Discrete')then
	    chr_VERT_SAMPLING_SCHEME(3) = 'Secondary sampling: discrete []';
        elseif(index(value,"Near-surface sampling:") > 0)then
            chr_VERT_SAMPLING_SCHEME(3) = trim(value);  ! these are complete lines, including the "near-surface" part
	elseif(index(value,'n/a') > 0)then
	    chr_VERT_SAMPLING_SCHEME(3) = 'Secondary sampling: N/A';
	else
	    chr_VERT_SAMPLING_SCHEME(3) = 'Secondary sampling: mixed []';
	endif
    else if(trim(variable) == 'Profile Sampling Method(4)')then
        sampling_found = .true.;  sampling_entries = sampling_entries + 1;
        VERT_SAMPLING_SCHEME_FROM = "META_FILE";
        if(index(value,'mixed [') > 0)then
            chr_VERT_SAMPLING_SCHEME(4) = 'Secondary sampling: '//trim(value);
        elseif(index(value,'discrete [') > 0)then
            chr_VERT_SAMPLING_SCHEME(4) = 'Secondary sampling: '//trim(value);
        elseif(index(value,'averaged [') > 0)then
            chr_VERT_SAMPLING_SCHEME(4) = 'Secondary sampling: '//trim(value);
        elseif(trim(value)=='continuous'.or.trim(value)=='Continuous')then
            chr_VERT_SAMPLING_SCHEME(4) = 'Secondary sampling: averaged []';
        elseif(trim(value)=='discrete'.or.trim(value)=='Discrete')then
            chr_VERT_SAMPLING_SCHEME(4) = 'Secondary sampling: discrete []';
        elseif(index(value,"Near-surface sampling:") > 0)then
            chr_VERT_SAMPLING_SCHEME(4) = trim(value);  ! these are complete lines, including the "near-surface" part
        elseif(index(value,'n/a') > 0)then
            chr_VERT_SAMPLING_SCHEME(4) = 'Secondary sampling: N/A';
        else
            chr_VERT_SAMPLING_SCHEME(4) = 'Secondary sampling: mixed []';
        endif
    else if(index(value,"STS") > 0)then
	STS_prof(0) = .true.;
	if(trim(variable) == "conductivity(2) sensor type")then
	    STS_prof(2) = .true.;
	elseif(trim(variable) == "conductivity(3) sensor type")then
	    STS_prof(3) = .true.;
	elseif(trim(variable) == "conductivity(4) sensor type")then
	    STS_prof(4) = .true.;
	endif
    else if(trim(variable) == 'send to GTS')then
	send_GTS_found = .true.
	if(value(1:1) == "n")then
	    send_to_GTS = .false.
	endif
    else if(trim(variable) == 'add to grey list')then
	grey_list_found = .true.
	add_to_grey_list = .true.
	if(len(trim(value)) == 1)then
	    ngl = 1;					! The umber of grey list entries defaults to 1
	else
	    testch = value(3:4); read(testch,*) ngl;	! The umber of grey list entries
	endif
	if(DBLEV==15)then; write(msg,'("GREYLIST There are ",i1," entries")')ngl; call wrtdbg(sub,msg); endif;
	if(ngl > MAX_GL)then
	    write(errmsg(1),'("FATAL ERROR 931 TOr MANY GREY LIST RECS at 2")');
	    write(errmsg(2),'("FATAL ERROR 931 Increase MAX_GL in data_arrays to at least ",i2)')ngl;
	    write(errmsg(3),'("FATAL ERROR 931 & add section to read 2ch n_grey_list")');
	    call error_exit(sub,931,3,errmsg); ! read_meta - too many grey list recs
	endif
	do igl=1,ngl
	    read(INP,'(a)',iostat=ios) cline; read(cline,'(a40,a)')variable,value	! Read DATE beg/end line
	    nmet=nmet+1; metline(nmet) = trim(cline);
	    grey_list_beg(igl)=value(1:10); grey_list_end(igl)=value(12:21);	! read beg/end from cline
	    read(INP,'(a)',iostat=ios) cline; read(cline,'(a40,a)')variable,value	! Read flag value line
	    nmet=nmet+1; metline(nmet) = trim(cline);
	    testst = value(1:40);			! ngp - number of parameters for this entry
	         if(testst(7:7) /= " ")then;ngp=4;
	    else if(testst(5:5) /= " ")then;ngp=3;
	    else if(testst(3:3) /= " ")then;ngp=2;
	    else ;  ngp=1;endif
	    read(testst,*)(test_flags(igp),igp=1,ngp)
	    if(DBLEV==15)then; write(msg,'("GREYLIST Entry: ",i1," has ",i1," parameters")')igl,ngp; call wrtdbg(sub,msg); endif;
	    read(INP,'(a)',iostat=ios) cline; read(cline,'(a40,a)')variable,value ! parameter line
	    nmet=nmet+1; metline(nmet) = trim(cline);
	    testst = value(1:40); read(testst,'(a4,1x,a4,1x,a4,1x,a4)')(test_params(igp),igp=1,ngp);
	    do igp=1,ngp
		     if(test_params(igp) == "PRES")then;param_on_grey_list(igl,1)=.true.;grey_flag(igl,1)=test_flags(igp);
		else if(test_params(igp) == "TEMP")then;param_on_grey_list(igl,2)=.true.;grey_flag(igl,2)=test_flags(igp);
		else if(test_params(igp) == "PSAL")then;param_on_grey_list(igl,3)=.true.;grey_flag(igl,3)=test_flags(igp);
		else if(test_params(igp) == "DOXY")then;param_on_grey_list(igl,4)=.true.;grey_flag(igl,4)=test_flags(igp);
		else
		    write(errmsg(1),'("FATAL ERROR 932 READING GREYLIST PARAMS at 3")');
		    write(errmsg(2),'("FATAL ERROR 932 Param ",i1, " tested is ",a)') igp,test_params(igp);
		    write(errmsg(3),'("FATAL ERROR 932  Param must be one of PRES TEMP PSAL DOXY")');
		    call error_exit(sub,932,3,errmsg); ! read_meta - error reading greylist parameters
	 	endif
	    enddo
	    read(INP,'(a)',iostat=ios) cline; ! reason line - just throw it away
	    nmet=nmet+1; metline(nmet) = trim(cline);
	enddo
    else if(trim(variable)=='PSAL ADJUST [dd mm yyyy N S_off stddev]')then
	psal_adj_found=.true.; 
	read(cline,'(a40,a)')variable,value
	npa = npa+1;
	if(npa > MAX_PA)then
	    write(errmsg(1),'("FATAL ERROR 933 TOO MANY PSAL_ADJ LIST RECS ")');
	    write(errmsg(2),'("FATAL ERROR 933 Increase MAX_PA in data_arrays to at least ",i2)')npa;
	    call error_exit(sub,933,3,errmsg); ! read_meta - need to increse max params in data_arrays
	endif
	psal_adj_line(npa)=trim(value) ! store the lines to analyze later
    else if(trim(variable) == "transmission type")then
	trans_type_found = .true.
	trans_type = trim(value);
	chr_POSITIONING_SYSTEM = trim(trans_type);
	POSITIONING_SYSTEM_from = 'POSITIONING_SYSTEM FROM                 META_FILE TRANSMISSION TYPE'
	if(DBLEV==49)then; 
	    write(msg,'("read_meta Set chr_POSITIONING_SYSTEM=",a," (transmission type)")')trim(trans_type); ! LEV49
	    call wrtdbg(sub,msg);
	endif
    else if(trim(variable) == "Project name" .or. trim(variable) == "project name")then
	chr_PROJECT_NAME = trim(value);
    else if(trim(variable) == "FROZEN PROFILE RULE")then
	read(value,*)frozen_profile_rule
    else if(trim(variable) == "up time [hours]")then
	UP_TIME_found = .true.;
	read(value,*)UP_TIME_HOURS;
	UP_TIME_DAYS = UP_TIME_HOURS/24.0;
    else if(trim(variable) == "cycles for profile pressure")then
	read(cline,'(a40,a)')variable,value
	if(index(trim(value)," ") > 0)then;
	    read(value,*)nominal_prof_ncyc1,nominal_prof_ncyc2
	else
	    if(trim(value) == "n/a")then
		nominal_prof_ncyc2 = 0; nominal_prof_ncyc1 = 1;
	    else
		nominal_prof_ncyc2 = 0; read(value,*)nominal_prof_ncyc1
	    endif
	endif
    else if(trim(variable) == "nominal profile pressure [dbar]")then
	read(cline,'(a40,a)')variable,value
	nextra=0
	do icnt=1,len(trim(value))
	    if(value(icnt:icnt) == " ") nextra=nextra+1
	enddo
	if(nextra>1)then
	    write(errmsg(1),'("NON-FATAL ERROR 233 in read_meta")')
	    write(errmsg(2),'("NON-FATAL ERROR 233 more than two nominal profile pressures found in meta file")');
	    write(errmsg(3),'("NON-FATAL ERROR 233 value: ",a)')trim(value)
	    call error_report(sub,233,3,errmsg); ! read_meta more than two nominal profile pres
	endif
	if(nextra == 0)then
	    read(value,*) nominal_prof_pres;
	else
	    double_nominal_prof_pres = .true.
	    read(value,*) nominal_prof_pres1,nominal_prof_pres2
	    nominal_prof_pres = max(nominal_prof_pres1,nominal_prof_pres2); ! Always use the deepest pres for deepest pres test
	endif
	maximum_prof_pres = nominal_prof_pres;	! This is what is used for qc19 (unless changed by phy file)
	write(msg,'("DEEPEST_PRES In read_meta set maximum_prof_pres: ",f12.2)')maximum_prof_pres; call wrtlog(sub,msg);
    else if(trim(variable) == "nominal drift pressure [dbar]")then
	read(cline,'(a40,a)')variable,value
	nextra=0
	do icnt=1,len(trim(value))
	    if(value(icnt:icnt) == " ") nextra=nextra+1
	enddo
	if(nextra>1)then
	    write(errmsg(1),'("NON-FATAL ERROR 234 in read_meta")')
	    write(errmsg(2),'("NON-FATAL ERROR 234 More than two nominal drift pressures found in meta file")');
	    write(errmsg(3),'("NON-FATAL ERROR 234 value: ",a)')trim(value)
	    call error_report(sub,234,3,errmsg); ! read_meta han two nominal drift pres
	endif
	if(nextra > 0)then
            double_nominal_drft_pres = .true.
        endif
    else if(INDEX(trim(variable), 'calib eq 1 for conductivity') > 0)then
            hasCNDC_meta=.true.;          ! means - it has both salinity and conductivity as parameters.
    endif
    10 continue; ! SKIP PARSING, line is =====s
    read(INP,'(a)',iostat=ios) cline
    nmet=nmet+1; metline(nmet) = trim(cline);
enddo
if(DEBUG)then
    if(trim(chr_VERT_SAMPLING_SCHEME(2)) .ne. "n/a") &
	write(DBG,*)'STS/VSS: ',STS_prof(2),' ',trim(chr_VERT_SAMPLING_SCHEME(2)) ! DBG
endif
if(DEBUG)then; write(msg,'("Meta file read completely")'); call wrtdbg(sub,msg); endif
inp_open=close_file(INP,"INP","meta file");

! 20160402 Hide the trans_id for IRIDIUM floats - can be used to break into the phone number
if(trim(trans_type) == "IRIDIUM")then; var_TRANSMISSION_NO = "n/a"; endif

! This should NEVER happen, so check to make sure
if(double_nominal_drft_pres)then
    if(double_nominal_prof_pres)then
	write(errmsg(1),'("FATAL ERROR 940 Both nom_prof_pres and nom_drivt_pres have double values")')
	write(errmsg(2),'("FATAL ERROR 940 This should NOT happen, we must have a corrupted meta fle")')
	call error_exit(sub,940,2,errmsg); ! FATAL ERROR 940 ! read_meta - mon pref pres & nom direct pres both doublevalued
    endif
endif

	

! The metafile has been read, make sure you have everything you need
! If any required fileds are missing, go to991

if(.not.  wmoid_found)       go to 991 ! READ_META ERROR - NOT ALL REQUIRED FIELDS WERE FOUND
if(.not.  inst_type_found)   go to 991 ! READ_META ERROR - NOT ALL REQUIRED FIELDS WERE FOUND
if(.not.  trans_type_found)  go to 991 ! READ_META ERROR - NOT ALL REQUIRED FIELDS WERE FOUND
if(.not.  launch_time_found) go to 991 ! READ_META ERROR - NOT ALL REQUIRED FIELDS WERE FOUND
if(.not.  launch_posn_found) go to 991 ! READ_META ERROR - NOT ALL REQUIRED FIELDS WERE FOUND
if(.not.  sampling_found)    go to 991 ! READ _METAERROR - NOT ALL REQUIRED FIELDS WERE FOUND

! Do we have a DEEP ARGO float? If so,what type?
if(DEBUG)then; write(msg,'("read_meta inst_type: ",a)')trim(inst_type); call wrtdbg(sub,msg); endif
if(inst_type(1:5) == "SOLOD")then;
    write(msg,'("read_meta inst_type is a DEEP ARGO float: ",a)')trim(inst_type); call wrttrc(sub,msg);
    DEEP_ARGO_FLOAT = .true.;
elseif(inst_type(1:5) == "APEXD")then;
    write(msg,'("read_meta inst_type is a DEEP ARGO float: ",a)')trim(inst_type); call wrttrc(sub,msg);
    DEEP_ARGO_FLOAT = .true.;
elseif(inst_type(1:6) == "NAVISD")then;
    msg = "INSTRUMENT TYPE indicates a DEEPO ARGO float that we can not yet handle"; call wrttrc(sub,msg);
    write(errmsg(1),'("FATAL ERROR 938 ",a," ",a)')trim(sub),trim(msg);
    write(errmsg(2),'("FATAL ERROR 938 inst_type: ",a)')trim(inst_type);
    call error_exit(sub,938,2,errmsg); ! FATAL ERROR 938 ! read_meta - cannot yet handle NAVISD float
endif

! get the short instruemnt_type for the report line (set to lower)
ib = index(inst_type,"_")+1;
inst_type_short = inst_type(ib:ib+4);
if(inst_type_short(1:3)=="TSO") inst_type_short(1:3)="tso";
if(inst_type_short(1:2)=="TS") inst_type_short(1:2)="ts";
if(inst_type_short(1:1)=="T") inst_type_short(1:1)="t";

! The metafile has been read, and you have everything you need

! here is where we deterin the status of pres & psal adjustment
! First, do we need to consider pres adjustment
! Do NOT adjust SOLOIR

pres_adj_comment = "PRES ADJUSTMENT NOT REQUIRED INST_TYPE = " //trim(inst_type);

if(inst_type(1:7) == 'APEX_T3')then    
    try_pres_adj = .false.; do_pres_adj = .false.; ! DBGPRESADJ read_meta set adjust=F inst=APEX_T3         
elseif(inst_type(1:4) == 'APEX')then   
    try_pres_adj = .true.;  do_pres_adj = .true.;  ! DBGPRESADJ read_meta set adjust=T inst=APEX not T3     
    pres_adj_comment = "PRES ADJUSTMENT TO BE DETERMINED INST_TYPE = " //trim(inst_type);
elseif(inst_type(1:5) == "SOLOD")then  
    try_pres_adj = .true. ; do_pres_adj = .true.;  ! DBGPRESADJ read_meta set adjust=T inst=SOLOD (not SOLO)
    pres_adj_comment = "PRES ADJUSTMENT TO BE DETERMINED INST_TYPE = " //trim(inst_type);
elseif(inst_type(1:4) == "SOLO")then   
    try_pres_adj = .false.; do_pres_adj = .false.; ! DBGPRESADJ read_meta set adjust=F inst=SOLO (not SOLOD)
elseif(inst_type(1:7) == 'NAVISIR')then
    try_pres_adj = .true.;  do_pres_adj = .true.;  ! DBGPRESADJ read_meta set adjust=T inst=NAVISIR         
    pres_adj_comment = "PRES ADJUSTMENT TO BE DETERMINED INST_TYPE = " //trim(inst_type);
elseif(inst_type(1:6) == 'PALACE')then 
    try_pres_adj = .false.; do_pres_adj = .false.; ! DBGPRESADJ read_meta set adjust=F inst=PALACE          
elseif(inst_type(1:6) == 'PROVOR')then 
    try_pres_adj = .false.; do_pres_adj = .false.; ! DBGPRESADJ read_meta set adjust=F inst=PROVOR          
elseif(inst_type(1:7) == 'ALAMOIR')then
    try_pres_adj = .false.; do_pres_adj = .false.; ! DBGPRESADJ read_meta set adjust=F inst=ALAMOIR
elseif(inst_type(1:6) == 'ALTOIR')then
    try_pres_adj = .false.; do_pres_adj = .false.; ! DBGPRESADJ read_meta set adjust=F inst=ALTOIR
else
    write(msg,'("The inst_type was NOT recognized, issue a FATAL ERROR: ",a)')trim(inst_type); call wrtlog(sub,msg);
    call exit(4); ! SAV Failed to identify the inst_type correctly. Just get outta Dodge for now
endif
if(DEBUG)then; write(msg,*)'  inst_type: ',trim(inst_type); call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,*)'Do pres adj: ',do_pres_adj; call wrtdbg(sub,msg); endif
! Now, do we need the salinity adjustment?
if(psal_adj_found)then
    ! use the cycle number to determine
    ! work our way BACKWARDS throught the psal adj records
    do i=npa,1,-1
	read(psal_adj_line(i),*) psal_adj_dd, psal_adj_mm,psal_adj_yr,psal_adj_cyc,psal_offset,psal_stddev
	if(thiscyc >= psal_adj_cyc)then;
	    ! one more check = is the adjustment large enough?
	    ! THE NEXT LINE FORCES PSAL ADJ 
	    ! if(DEBUG)then; psal_offset=0.001; write(msg,*)'FAKE PSAL OFFSET, use : ',psal_offset; call wrtdbg(sub,msg); endif
	    if(abs(psal_offset) < 0.0005)then
		do_psal_adj = .false. ! it is already at false, but just make sure
		write(msg,'("PSAL_ADJUSTMENT will NOT be performed: ",f8.4," (abs(adjustment)< 0.0005 threshold)")')psal_offset;
		ntrcmsg = ntrcmsg + 1; trcmsg(ntrcmsg) = trim(msg);
	    else
		do_psal_adj = .true.; ! The psal_offset is large enough to use
		psal_adj_used = trim(psal_adj_line(i))
	    endif
	    go to 33;
	endif
    enddo
    33 continue
endif
! ==================================================================================================

read_meta = 0;                          ! Signal success

if(DBGIO) call dbg_leave(sub,read_meta);

return; ! Normal return

991 continue ! READ_META ERROR - NOT ALL REQUIRED FIELDS WERE FOUND
! not all the required fields were found, report an error and exit
write(errmsg(1),'("FATAL ERROR 939 READ_META ERR - NOT ALL REQUIRED FIELDS WERE FOUND")');
write(errmsg(2),'("FATAL ERROR 939  meta file: ",a)')trim(pathname);
write(errmsg(3),'("FATAL ERROR 939  The required variables are:")'); call wrttrc(sub,msg);
write(errmsg(4),*)'FATAL ERROR 939        WMOID_found: ',wmoid_found;       call wrttrc(sub,msg);
write(errmsg(5),*)'FATAL ERROR 939    inst_type_found: ',inst_type_found;   call wrttrc(sub,msg);
write(errmsg(6),*)'FATAL ERROR 939   trans_type_found: ',trans_type_found;  call wrttrc(sub,msg);
write(errmsg(7),*)'FATAL ERROR 939  launch_time_found: ',launch_time_found; call wrttrc(sub,msg);
write(errmsg(8),*)'FATAL ERROR 939  launch_posn_found: ',launch_posn_found; call wrttrc(sub,msg);
write(errmsg(9),*)'   sampling_found: ',sampling_found;    call wrttrc(sub,msg);
call error_exit(sub,939,9,errmsg); ! read_meta - not all required fields were found

end function read_meta
! ===========================================================================================================
